<?php
namespace Controllers; 

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\Controller;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Response\Notification;

class Gop01200ArrecadacaoAdministrativaController {
    protected $dataSourceManager;
    protected $connection;
    protected $newCode;
    protected $environment;
    protected $service;

    public function __construct($crudService,  $environment, $dataSourceManager, $connection, $newCode, $service) {
    
        $this->environment       = $environment;
        $this->dataSourceManager = $dataSourceManager;
        $this->connection        = $connection;
        $this->newCode           = $newCode;
        $this->service           = $service;
    }
    
    
    public function saveUnit (DTO\Request $request, DTO\Response $response) {
        $rows = $request->getDataSet()->getRows();
        
       try{
            $this->service->beginTransaction();
    
            foreach($rows as $row) {
                $params = array(
                   'P_CDFILIAL' => $row['CDFILIAL'],
                   'P_NRORG' => $this->environment->getNrorg()
                );
                $this->service->saveUnit($params);
            }
        $this->service->commit();
        }catch(\Exception $ex) {
            $this->service->rollback();
            $response->setError(new Error("Não foi possível processar os dados enviados" . $ex->getMessage(), 500));
        }

    }
     public function saveUnityGeneral (DTO\Request $request, DTO\Response $response) {
        $rows = $request->getDataSet()->getRows();
        try{
            $this->service->beginTransaction();
            foreach($rows as $row){
                $params = array(
                    'P_CDANO' => $row['CDANO'],
                    'P_CDFILIALADM' => $row['CDFILIALADM'],
                    'P_VRMES01'=> $row['VRMES01'],
                    'P_VRMES02'=> $row['VRMES02'],
                    'P_VRMES03'=> $row['VRMES03'],
                    'P_VRMES04'=> $row['VRMES04'],
                    'P_VRMES05'=> $row['VRMES05'],
                    'P_VRMES06'=> $row['VRMES06'],
                    'P_VRMES07'=> $row['VRMES07'],
                    'P_VRMES08'=> $row['VRMES08'],
                    'P_VRMES09'=> $row['VRMES09'],
                    'P_VRMES10'=> $row['VRMES10'],
                    'P_VRMES11'=> $row['VRMES11'],
                    'P_VRMES12'=> $row['VRMES12'],
                    'P_NRORG' => $this->environment->getNrorg());
                    
                $this->service->saveUnityGeneral($params);
                
                $paramsSelect = array(
                    'P_CDANO' => $row['CDANO'],
                    'P_NRORG' => $this->environment->getNrorg()
                );
                $error = $this->service->checkRowsMonthGridGeneral($paramsSelect);
            }
            if(!empty($error)){
                $this->service->rollback();
                $response->addDataSet(new DataSet("data", $error));
            } else{
               $this->service->commit();  
            } 
        }catch(\Exception $e) {
            $this->service->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function saveUnityGrid (DTO\Request $request, DTO\Response $response) {
        $rows = $request->getDataSet()->getRows();
        try {
            $this->service->beginTransaction();
            foreach($rows as $row){ 
                $params = array(
                        'P_CDANO' => $row['CDANO'],
                        'P_CDFILIAL' => $row['CDFILIAL'],
                        'P_CDFILIALADM' => $row['CDFILIALADM'],
                        'P_VRMES01'=> $row['VRMES01'],
                        'P_VRMES02'=> $row['VRMES02'],
                        'P_VRMES03'=> $row['VRMES03'],
                        'P_VRMES04'=> $row['VRMES04'],
                        'P_VRMES05'=> $row['VRMES05'],
                        'P_VRMES06'=> $row['VRMES06'],
                        'P_VRMES07'=> $row['VRMES07'],
                        'P_VRMES08'=> $row['VRMES08'],
                        'P_VRMES09'=> $row['VRMES09'],
                        'P_VRMES10'=> $row['VRMES10'],
                        'P_VRMES11'=> $row['VRMES11'],
                        'P_VRMES12'=> $row['VRMES12'],
                        'P_NRORG' => $this->environment->getNrorg());
                        
                $this->service->saveUnityGrid($params);
                
                $paramSelect = array(
                    'P_CDFILIAL' => $row['CDFILIAL'],
                    'P_CDANO' => $row['CDANO'],
                    'P_NRORG' => $this->environment->getNrorg());
                    
                $mes = 'T';
                $error = $this->service->checkRowsMonthGridUnity($paramSelect, $mes);   
            }
            if(!empty($error)){
                $this->service->rollback();
                $response->addDataSet(new DataSet("data", $error));
            } else{
               $this->service->commit();  
            } 
        }catch(\Exception $e) {
            $this->service->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function deleteRows(DTO\Request $request, DTO\Response $response) {
        $rows = $request->getDataSet()->getRows();
        foreach($rows as $row){
            $params = array(
               'P_CDFILIALADM' => $row['CDFILIALADM'],
            );
            $query = "UPDATE ARRECADAADMGER SET IDATIVO = 'N', VRMES01 = null, VRMES02 = null, VRMES03 = null, VRMES04 = null, VRMES05 = null, VRMES06 = null, VRMES07 = null, VRMES08 = null, VRMES09 = null, VRMES10 = null, VRMES11 = null, VRMES12 = null WHERE CDFILIALADM = :P_CDFILIALADM";
            
            try {
                $stmt = $this->connection->prepare($query);
                $stmt->execute($params);
            } catch (Exception $e) {
                $response->setError(new Error($e->getMessage(), 500));
            }
        }
    }
    
    public function criarListaValores($lista){
        $this->service->beginTransaction();
        $nrlista = $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrorgTrab(), 1, 10, false);
        try{
            foreach($lista as $value){
                $this->insertLista($nrlista, $value);
            }
            $this->service->commit();
            return $nrlista;
        } catch(\Exception $e){
            $this->service->rollback();
            throw $e;
        }
    }
    
    public function insertLista($nrlista, $cdlista) {
        $params = array(
            "NRLISTA" => $nrlista,
            "CDLISTA" => $cdlista
        );
        
        $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA) VALUES (:NRLISTA, :CDLISTA)";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    public function cadAutomatic(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        
        try {
            $listUnd = $this->criarListaValores($row['CDFILIAL']);
            $listUndAdm = $this->criarListaValores($row['CDFILIALADM']);
            $this->service->beginTransaction();
     
            $mes = "";
            $paramMeses = ""; 
            $mesUpdate = "";
            
            foreach($row["MESES"] as $indice => $nrMes){
                $mes = $mes . "VRMES" . $nrMes["value"] . " ,";
                $paramMeses = $paramMeses . ":P_VRMES, ";
                $mesUpdate = ($indice != 0) ? $mesUpdate . ", " : $mesUpdate;
                $mesUpdate = $mesUpdate . "VRMES". $nrMes["value"] . " = :P_VRMES";
            }
            $params = array(
                "P_CDANO" => $row['CDANO'],
                "P_NRLISTA"=> $listUnd,
                "P_CDFILIALADM"=> $listUndAdm,
                "P_VRMES" => $row["PERCENTUALADM"], 
                "P_NRORG" => $this->environment->getNrorg()
            );
            
            $this->service->cadAutomatic($params, $mes, $paramMeses, $mesUpdate);
            
            $paramSelect = array(
                "P_CDANO" => $row['CDANO'],
                "P_NRLISTA"=> $listUnd,
                "P_NRORG" => $this->environment->getNrorg()
            );
            $error = $this->service->checkRowsMonthCadAutomaic($paramSelect, $mes);
            
            if(!empty($error)){
                $this->service->rollback();
                $response->addDataSet(new DataSet("data", $error));
            } else{
               $this->service->commit();  
            }
        }catch(\Exception $e) {
            $this->service->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
}